/* -*- c -*-
 * Time-stamp: <2005-04-23 20:18:13 rsmith>
 * 
 * onepad/padfile.c
 * Copyright (C) 2001--2005 R.F. Smith <rsmith@xs4all.nl>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "padfile.h"

#ifndef NULL
#define NULL (void*)0
#endif

#ifndef RETURN_IF_FAIL
#define RETURN_IF_FAIL(test) if (!(test)) return
#endif

#ifndef RETURN_VAL_IF_FAIL
#define RETURN_VAL_IF_FAIL(test,val) if (!(test)) return ((val))
#endif

#ifndef NDEBUG
/* __FUNCTION__ is a GCC feature. */
# ifdef __GNUC__
#  undef debug
#  define debug(TXT) fprintf(stderr,"%s, line %i, %s(): %s\n",\
__FILE__,__LINE__,__FUNCTION__,TXT)
# else
#  define debug(TXT) fprintf(stderr,"%s, line%i: %s\n",__FILE__,__LINE__,TXT)
# endif
#else
# undef debug
# define debug(TXT) (void)0
#endif				/* NDEBUG */

/* This enables you to see which files have been used in building a
 * binary. N.B, __attribute is a GCC extension, so we test for __GNUC__. */
#ifdef __GNUC__
static char *SCMID __attribute__ ((unused)) = 
"$Id: padfile.c 43 2005-04-23 18:19:09Z rsmith $";
#endif

/**
 * pfopenr - open a key or cyphertext file for reading
 * @name: path of the file to open
 * 
 * Opens a file with the given @name, and tries to read the file ID.
 * Returns a struct padfile on success, and NULL on failure.
 **/
struct padfile *pfopenr(const char *name)
{
	struct padfile *pf = NULL;
	
	assert(name);
	pf = calloc(1, sizeof(struct padfile));
	if (pf == NULL) {
		debug("cannot allocate memory for struct padfile!");
		return NULL;
	}
	pf->f = fopen(name, "r");
	if (pf->f == NULL) {
		debug("cannot open padfile for reading!");
		free(pf);
		return NULL;
	}
	if (fscanf(pf->f, "%63s", pf->id) != 1) {
		debug("cannot read padfile identifier!");
		free(pf);
		return NULL;		
	}
	pf->read = 1;
	return pf;
}

/**
 * pfopenw - open a key or cyphertext file for writing
 * @name: path of the file to open
 * @id: key identifier
 * 
 * Opens a file with the given @name, and tries to write the file ID.
 * Returns a struct padfile on success, and NULL on failure.
 **/
struct padfile *pfopenw(const char *name, const char *id)
{
	struct padfile *pf = NULL;
	int rv;
	
	assert(name);
	pf = calloc(1, sizeof(struct padfile));
	if (pf == NULL) {
		debug("cannot allocate memory for struct padfile!");
		return NULL;
	}
	strncpy(pf->id, id, 63);
	pf->f = fopen(name, "w");
	if (pf->f == NULL) {
		debug("cannot open padfile for writing!");
		free(pf->id);
		free(pf);
		return NULL;
	}
	rv = fprintf(pf->f, "%s\n\n", pf->id);
	if (rv != strlen(pf->id)+2){
		debug("cannot write padfile ID!");
		free(pf->id);
		free(pf);
		return NULL;
	}
	pf->ln = 2;
	pf->col = 1;
	return pf;
}

/**
 * pfreadb - read a single byte from a key or cyphertext file.
 * @pf: key or cyphertext file structure
 * @pb: location where the read byte is stored
 * 
 * Reads a single byte from a key or cyphertext file, and stores it at the
 * location pointed to by @pb. Returns 0 on normal completion, and !=0 in
 * case of an error.
 **/
int pfreadb(struct padfile *pf, unsigned char *pb)
{
	int rv;
	unsigned int scanval;
	assert(pf);
	assert(pb);
	if (pf->read == 0) {
		debug("padfile in write mode!");
		return 1;
	}
	rv = fscanf(pf->f, " %2X", &scanval);
	if (rv != 1) {
		return 2;
	}
	*pb = (unsigned char) scanval;
	return 0;
}

/**
 * pfwritedb - writes a single byte to a key or cyphertext file.
 * @pf: key or cyphertext file structure
 * @b: byte to write to the file.
 * 
 * Writes a single byte to a key or cyphertext file, converting it to a
 * nicely formatted hexadecimal notation. Returns 0 on normal completion,
 * and !=0 in case of an error.
 **/
int pfwriteb(struct padfile *pf, unsigned char b)
{
        char *patterns[4] = 
		{
			"%02X", "%02X ", "%02X\n", "%02X\n\f\n"
		};
	char *pat = patterns[0];
	int rv;
	assert(pf);
	if (pf->read == 1) {
		debug("padfile in read mode!");
		return 1;
	}
	if (pf->col > 0) {
		if (pf->col % 5 == 0) {
			pat = patterns[1];
		}
		if (pf->col % (5*7) == 0) {
			pat = patterns[2];
			pf->col = 0;
			pf->ln++;
		}
	}
	if (pf->ln > 66) {
		pat = patterns[3];
		pf->ln = 0;
		pf->col = 0;
	}
	rv = fprintf(pf->f, pat, (unsigned int)b);
	pf->col++;
 	if (rv != strlen(pat)-2) {
 		debug("error writing hex value!");
 		return 2;
 	}
	return 0;
}

/**
 * pfclose - closes a key or cyphertext file.
 * @pf: key or cyphertext file structure
 * 
 * Closes the file, and frees all the associated resources. The pointer @pf
 * is invalid after this call.
 **/
int pfclose(struct padfile *pf)
{
	assert(pf);
	fclose(pf->f);
	free(pf);
	return 0;
}

/* EOF padfile.c */
